<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 13:37
 */

namespace top\base;

use ArrayAccess;
use Closure;
use Countable;
use ReflectionClass;
use ReflectionException;
use ReflectionFunction;
use ReflectionMethod;
use ReflectionParameter;
use top\exceptions\app\CallFailed;
use top\exceptions\app\ClassNotFound;

/**
 * 容器类
 * Class Container
 * @package top\base
 */
class Container implements ArrayAccess, Countable
{
    /**
     * 当前实例
     * @var null|Container
     */
    protected static ?Container $instance = null;

    /**
     * 实例池
     * @var array
     */
    protected array $instances = [];

    /**
     * 绑定标识
     * @var array
     */
    protected array $bind = [];

    /**
     * 获取当前实例
     * @return static
     */
    public static function getInstance(): static
    {
        if (is_null(static::$instance)) {
            static::$instance = new static();
        }

        return static::$instance;
    }

    /**
     * 设置当前实例
     * @param Container $instance
     */
    protected static function setInstance(Container $instance): void
    {
        static::$instance = $instance;
    }

    /**
     * 绑定标识到容器
     * @param string $name
     * @param callable|object|string $bind
     * @return $this
     */
    public function bind(string $name, callable|object|string $bind): static
    {
        if (!isset($this->bind[$name])) {
            $this->bind[$name] = $bind;
        }

        return $this;
    }

    /**
     * 绑定实例到容器
     * @param string $name
     * @param object $bind
     * @return $this
     */
    public function instance(string $name, object $bind): static
    {
        $name = $this->getAlias($name);

        $this->instances[$name] = $bind;

        return $this;
    }

    /**
     * 获取别名
     * @param string $name
     * @return string
     */
    public function getAlias(string $name): string
    {
        if (isset($this->bind[$name])) {
            $bind = $this->bind[$name];

            if (is_string($bind)) {
                return $this->getAlias($bind);
            }
        }

        return $name;
    }

    /**
     * 获取类实例
     * @param string $name
     * @param array $args
     * @param bool $newInstance
     * @return mixed
     * @throws CallFailed
     * @throws ClassNotFound
     */
    public function make(string $name, array $args = [], bool $newInstance = false): mixed
    {
        // 获取到最终类名
        $class = $this->getAlias($name);

        // 检查类是否已经实例化，已经实例化则直接返回（需要新的实例则跳过此步）
        if (isset($this->instances[$class]) && !$newInstance) {
            return $this->instances[$class];
        }
        // 如果标识绑定到了闭包，则直接执行
        if (isset($this->bind[$class]) && $this->bind[$class] instanceof Closure) {
            $instance = $this->invokeFunction($this->bind[$class], $args);
        } else {
            // 非闭包则表示当前类名从未进行实例化，这里进行实例化
            $instance = $this->createInstance($class, $args);
        }

        // 加入到实例池
        if (!$newInstance && is_object($instance)) {
            $this->instance($class, $instance);
        }

        return $instance;
    }

    /**
     * 创建类实例
     * @param string $className
     * @param array $args
     * @return object
     * @throws CallFailed
     * @throws ClassNotFound
     */
    private function createInstance(string $className, array $args = []): object
    {
        try {
            $reflectionClass = new ReflectionClass($className);
        } catch (ReflectionException) {
            throw new ClassNotFound(sprintf("类 '%s' 不存在", $className));
        }
        if (!$reflectionClass->isInstantiable()) {
            throw new CallFailed(sprintf("类 '%s' 不能被实例化", $className));
        }
        // 检查是否存在构造方法，没有则直接返回实例
        $construct = $reflectionClass->getConstructor();
        if (is_null($construct)) {
            return new $className;
        }
        // 准备调用参数进行注入
        $parameters     = $construct->getParameters();
        $callParameters = $this->buildParameters($parameters, $args);

        // 返回实例
        try {
            return $reflectionClass->newInstanceArgs($callParameters);
        } catch (ReflectionException) {
            throw new CallFailed(sprintf("构建类 '%s' 实例失败", $className));
        }
    }

    /**
     * 构建调用参数
     * @param array|ReflectionParameter $reflectionParameter
     * @param array $args
     * @return array
     * @throws CallFailed
     */
    private function buildParameters(array|ReflectionParameter $reflectionParameter, array $args = []): array
    {
        $index          = 0;
        $callParameters = [];
        /** @var ReflectionParameter $parameter */
        foreach ($reflectionParameter as $parameter) {
            $type = (string)$parameter->getType();
            $name = $parameter->getName();
            try {
                // 一、明确指定参数
                // 1、如果在当前主动调用参数明确指定了参数，则直接使用
                // 二、未明确指定参数
                // 1、如果是类，则判断主动调用参数是否符合约定类型
                //    是则直接使用，否则实例化一个新的对象作为参数
                // 2、不是类则按顺序赋值使用
                // 3、不是类且未主动传递参数则直接取默认值
                if (isset($args[$name])) {
                    $callParameters[$name] = $args[$name];
                } else {
                    // 参数是一个类实例
                    if (class_exists($type)) {
                        // 当前主动调用参数是当前约定类型则使用，否则直接构造
                        $instance = $this->make($type);
                        if (isset($args[$index]) && ($args[$index] instanceof $instance)) {
                            $callParameters[$name] = $args[$index];
                            $index++;
                        } else {
                            $callParameters[$name] = $instance;
                        }
                    } elseif (isset($args[$index])) {
                        $callParameters[$name] = $args[$index];
                        $index++;
                    } else {
                        // 如果不是类实例或没有主动调用参数则使用默认值
                        $callParameters[$name] = $parameter->getDefaultValue();
                    }
                }
            } catch (ReflectionException) {
                // 没有默认值
                $class  = $parameter->getDeclaringClass()->getName();
                $method = $parameter->getDeclaringFunction()->getName();
                throw new CallFailed(sprintf("'%s::%s()' 缺少 '%s' 参数", $class, $method, $name));
            } catch (ClassNotFound) {
                throw new CallFailed(sprintf("类 '%s' 不存在", $type));
            }
        }

        return $callParameters;
    }

    /**
     * 执行函数
     * @param mixed $function
     * @param array $args
     * @return mixed
     * @throws CallFailed
     */
    public function invokeFunction(mixed $function, array $args = []): mixed
    {
        try {
            $funcRef = new ReflectionFunction($function);
            // 自动注入参数
            $args = $this->buildParameters($funcRef->getParameters(), $args);
            return $funcRef->invokeArgs($args);
        } catch (ReflectionException) {
            throw new CallFailed("函数调用失败");
        }
    }

    /**
     * 执行类方法
     * @param string|object $className
     * @param string $methodName
     * @param array $args
     * @return mixed
     * @throws CallFailed
     * @throws ClassNotFound
     */
    public function invokeMethod(string|object $className, string $methodName, array $args = []): mixed
    {
        $className = is_object($className) ? $className::class : $this->getAlias($className);
        $instance  = $this->make($className);
        if (!method_exists($instance, $methodName)) {
            throw new CallFailed(sprintf("方法 '%s::%s()' 不存在", $className, $methodName));
        }
        try {
            $ref = new ReflectionMethod($className, $methodName);
            // 自动注入参数
            $args = $this->buildParameters($ref->getParameters(), $args);
            return $ref->invokeArgs($instance, $args);
        } catch (ReflectionException) {
            throw new CallFailed(sprintf("方法 '%s::%s()' 调用失败", $className, $methodName));
        }
    }

    /**
     * 使用成员变量方式获取实例
     * @param string $name
     * @return mixed
     * @throws CallFailed
     * @throws ClassNotFound
     */
    public function __get(string $name): mixed
    {
        if (isset($this->bind[$name])) {
            return $this->make($name);
        }
        return null;
    }

    /**
     * 绑定实例
     * @param string $name
     * @param $value
     */
    public function __set(string $name, $value): void
    {
        $this->bind($name, $value);
    }

    /**
     * 设置绑定标识
     * @param string $name
     * @param mixed $value
     */
    public function set(string $name, mixed $value): void
    {
        $this->bind($name, $value);
    }

    /**
     * 检查是否存在绑定标识
     * @param string $name
     * @return bool
     */
    public function has(string $name = ''): bool
    {
        return isset($this->bind[$name]);
    }

    /**
     * 移除绑定标识
     * @param string $name
     */
    public function remove(string $name): void
    {
        $alias = $this->getAlias($name);
        unset($this->instances[$alias]);
        unset($this->bind[$name]);
    }

    /**
     * 检查是否存在绑定标识
     * @param mixed $offset
     * @return bool
     */
    public function offsetExists(mixed $offset): bool
    {
        return $this->has($offset);
    }

    /**
     * 获取实例
     * @param mixed $offset
     * @return mixed
     * @throws CallFailed
     * @throws ClassNotFound
     */
    public function offsetGet(mixed $offset): mixed
    {
        return $this->make($offset);
    }

    /**
     * 绑定标识
     * @param mixed $offset
     * @param mixed $value
     */
    public function offsetSet(mixed $offset, mixed $value): void
    {
        $this->bind($offset, $value);
    }

    /**
     * 移除标识
     * @param mixed $offset
     */
    public function offsetUnset(mixed $offset): void
    {
        $this->remove($offset);
    }

    /**
     * 返回容器中实例数量
     * @return int
     */
    public function count(): int
    {
        return count($this->instances);
    }

}
