<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 14:48
 */

namespace top\base;

use JetBrains\PhpStorm\Pure;
use top\base\response\Html;
use top\base\response\Redirect;
use top\base\response\Text;
use top\traits\Instance;

/**
 * 响应类
 * Class Response
 * @package top\base
 */
class Response
{
    /**
     * 当前响应类型
     * @var Response
     */
    private Response $type;

    /**
     * 响应header
     * @var array
     */
    protected array $headers = [];

    /**
     * 响应内容
     * @var string
     */
    protected string $content = '';

    /**
     * Http版本
     * @var string
     */
    protected string $httpLevel = 'HTTP/1.1';

    /**
     * Http状态码
     * @var int
     */
    protected int $httpCode;

    /**
     * Http状态码详情
     * @var array
     */
    protected array $httpStatus = [
        100 => 'Continue',
        101 => 'Switching Protocols',
        200 => 'OK',
        201 => 'Created',
        202 => 'Accepted',
        203 => 'Non-Authoritative Information',
        204 => 'No Content',
        205 => 'Reset Content',
        206 => 'Partial Content',
        300 => 'Multiple Choices',
        301 => 'Moved Permanently',
        302 => 'Moved Temporarily ',
        303 => 'See Other',
        304 => 'Not Modified',
        305 => 'Use Proxy',
        307 => 'Temporary Redirect',
        400 => 'Bad Request',
        401 => 'Unauthorized',
        402 => 'Payment Required',
        403 => 'Forbidden',
        404 => 'Not Found',
        405 => 'Method Not Allowed',
        406 => 'Not Acceptable',
        407 => 'Proxy Authentication Required',
        408 => 'Request Timeout',
        409 => 'Conflict',
        410 => 'Gone',
        411 => 'Length Required',
        412 => 'Precondition Failed',
        413 => 'Request Entity Too Large',
        414 => 'Request-URI Too Long',
        415 => 'Unsupported Media Type',
        416 => 'Requested Range Not Satisfiable',
        417 => 'Expectation Failed',
        500 => 'Internal Server Error',
        501 => 'Not Implemented',
        502 => 'Bad Gateway',
        503 => 'Service Unavailable',
        504 => 'Gateway Timeout',
        505 => 'HTTP Version Not Supported',
        509 => 'Bandwidth Limit Exceeded',
    ];

    /**
     * Response constructor.
     * @param int $httpCode
     * @param array $headers
     */
    public function __construct(int $httpCode = 200, array $headers = [])
    {
        $this->type     = $this;
        $this->httpCode = $httpCode;
        if (!empty($headers)) {
            $this->setHeaders($headers);
        }
    }

    /**
     * 设置Http状态码
     * @param int $httpCode
     * @return Response
     */
    public function setHttpCode(int $httpCode): Response
    {
        $this->type->httpCode = $httpCode;
        return $this->type;
    }

    /**
     * 获取Http状态码
     * @return int
     */
    public function getHttpCode(): int
    {
        return $this->type->httpCode;
    }

    /**
     * 设置Http协议版本
     * @param string $httpLevel
     * @return Response
     */
    public function setHttpLevel(string $httpLevel): Response
    {
        $this->type->httpLevel = $httpLevel;
        return $this;
    }

    /**
     * 获取Http协议版本
     * @return string
     */
    public function getHttpLevel(): string
    {
        return $this->type->httpLevel;
    }

    /**
     * 设置响应类型
     * @param string $type
     * @return $this
     */
    public function setType(string $type): Response|static
    {
        $this->type = app()->make($type);
        return $this->type;
    }

    /**
     * 获取响应类型
     * @return Response
     */
    public function getType(): Response
    {
        return $this->type;
    }

    /**
     * 设置当前响应header
     * @param array $headers
     * @return Response
     */
    public function setHeaders(array $headers = []): Response
    {
        $this->type->headers = array_merge($this->headers, $headers);
        return $this->type;
    }

    /**
     * 获取响应header
     * @return array
     */
    public function getHeaders(): array
    {
        return $this->type->headers;
    }

    /**
     * 设置响应内容
     * @param mixed $content
     * @return $this
     */
    public function setContent(mixed $content): static
    {
        $this->type->content = $content;
        return $this->type;
    }

    /**
     * 获取响应内容
     * @return string
     */
    public function getContent(): string
    {
        return $this->type->content;
    }

    /**
     * 响应header到客户端
     */
    private function outputHeaders(): void
    {
        $initHeader = $this->type->httpCode . ' ' . $this->type->httpStatus[$this->type->httpCode];
        $this->type->setHeaders([
            "{$this->type->httpLevel} $initHeader",
            "Status $initHeader",
        ]);
        foreach ($this->type->headers as $key => $header) {
            if (is_numeric($key)) {
                header($header);
            } else {
                header("$key: $header");
            }
        }
    }

    /**
     * 执行响应
     * @return string
     */
    public function send(): string
    {
        // 响应Header
        $this->type->outputHeaders();

        return $this->type->getContent();
    }

}
