<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 11:05
 */

namespace top\base;

use Closure;
use top\base\response\Redirect;
use top\exceptions\app\CallFailed;
use top\exceptions\app\ClassNotFound;
use top\exceptions\route\RouteNotFound;

/**
 * 路由
 * Class Route
 * @package top\base
 */
class Route
{
    /**
     * 当前组
     * @var array
     */
    private static array $currentGroupName = [];

    /**
     * 路由列表
     * @var array
     */
    private static array $routeList = [];

    /**
     * Route constructor.
     * @param Application $application
     * @param Request $request
     */
    public function __construct(
        private Application $application,
        private Request $request,
    )
    {
    }

    /**
     * GET
     * @param string $rule
     * @param mixed $route
     * @return Route
     */
    public static function get(string $rule, mixed $route): Route
    {
        return self::addRoute('get', $rule, $route);
    }

    /**
     * POST
     * @param string $rule
     * @param mixed $route
     * @return Route
     */
    public static function post(string $rule, mixed $route): Route
    {
        return self::addRoute('post', $rule, $route);
    }

    /**
     * PUT
     * @param string $rule
     * @param mixed $route
     * @return Route
     */
    public static function put(string $rule, mixed $route): Route
    {
        return self::addRoute('put', $rule, $route);
    }

    /**
     * DELETE
     * @param string $rule
     * @param mixed $route
     * @return Route
     */
    public static function delete(string $rule, mixed $route): Route
    {
        return self::addRoute('delete', $rule, $route);
    }

    /**
     * 添加路由到列表
     * @param string $type
     * @param string $rule
     * @param mixed $route
     * @return Route
     */
    private static function addRoute(string $type, string $rule, mixed $route): Route
    {
        $fullGroupName = implode('/', self::$currentGroupName);
        $rule          = ($rule != '/') ? trim($rule, '/') : $rule;
        $currentGroup  = ($fullGroupName) ? $fullGroupName . '/' . $rule : $rule;

        self::$routeList[$type][$currentGroup] = $route;

        return app()->route;
    }

    /**
     * 路由分组
     * @param string $name
     * @param callable $closure
     */
    public static function group(string $name, callable $closure): void
    {
        self::$currentGroupName[] = trim($name, '/');
        call_user_func($closure);
        array_pop(self::$currentGroupName);
    }

    /**
     * 获取当前访问方式的所有路由规则
     * @param $method
     * @return array
     */
    private function getRulesByMethod($method): array
    {
        return self::$routeList[$method] ?? [];
    }

    /**
     * 解析链接自带参数
     * @param string $url
     * @return array
     */
    private function parseUrlParameters(string $url): array
    {
        // 去除两头的/，保证匹配时和路由规则的统一
        $url           = ($url != '/') ? trim($url, '/') : $url;
        $parameters    = [];
        $urlComponents = parse_url($url);
        if (isset($urlComponents['query']) && $urlComponents['query']) {
            parse_str($urlComponents['query'], $parameters);
        }

        return [$urlComponents['path'], $parameters];
    }

    /**
     * 匹配路由规则
     * @return mixed
     */
    private function checkRoute(): mixed
    {
        list($url, $parameters) = $this->parseUrlParameters($this->request->url());
        // 获取当前请求方式的所有路由规则
        $rules       = $this->getRulesByMethod($this->request->getMethod());
        $currentRule = '';
        foreach ($rules as $rule => $route) {
            if (str_contains($rule, ':')) { // 规则表达式
                preg_match_all('#:\w+#', $rule, $keys);
                $pattern = preg_replace('#:[\d|\w]+#', '(((?!/).)*)', $rule);
                preg_match("#$pattern#", $url, $match);
                // 无匹配则直接开始下次匹配
                if (empty($match)) {
                    continue;
                }
                // 重建数组索引
                unset($match[0]);
                $match = array_merge($match, []);
                // 在匹配时第一次解析出参数
                $idx = 0;
                foreach ($keys[0] as $parameterName) {
                    $parameters[ltrim($parameterName, ':')] = $match[$idx];
                    $idx                                    += 2;
                }
            } else { // 正则表达式
                $pattern = "#^$rule$#";
                preg_match($pattern, $url, $match);
                if (empty($match[0])) {
                    continue;
                }
            }
            $currentRule = $route;
            break;
        }
        // 设置请求参数
        $this->request->setParameters($parameters);

        return $currentRule;
    }

    /**
     * 执行路由
     * @return Closure
     * @throws RouteNotFound
     */
    public function dispatch(): Closure
    {
        // 检查当前路由，获取到当前配置的路由信息
        $rule = $this->checkRoute();
        if ($rule == '') {
            throw new RouteNotFound(sprintf("请求 '%s' 无法处理", $this->request->url()));
        }
        // 打包应用
        return function () use ($rule) {
            $response = null;
            if (is_callable($rule)) {
                $response = $this->invokeFunction($rule);
            } elseif (is_array($rule)) { // 路由被定义为执行类方法
                $controller = $rule[0] ?? 'Index';
                $method     = $rule[1] ?? 'index';
                $response   = $this->invokeClass($controller, $method);
            } elseif (str_starts_with($rule, '/') || preg_match('/http[s]?:\/\//', $rule, $match)) {
                // TODO 重定向
                $response = $this->application->response
                    ->setType(Redirect::class)
                    ->setHttpCode(301)
                    ->setContent($rule);
            }

            // 如果路由处理结果为Response对象则直接返回，否则返回空的Response对象
            if ($response instanceof $this->application->response) {
                return $response;
            }

            return $this->application->response;
        };
    }

    /**
     * 类方法
     * @param string $name
     * @param string $method
     * @return mixed
     * @throws CallFailed
     * @throws ClassNotFound
     */
    private function invokeClass(string $name, string $method): mixed
    {
        // 解析路由规则携带的固定参数
        list($method, $fixedParameters) = $this->parseUrlParameters($method);
        if (!empty($fixedParameters)) {
            $this->request->setParameters($fixedParameters);
        }
        $requestParameters = $this->request->getParameters();
        // 调用类方法
        return $this->application->invokeMethod($name, $method, $requestParameters);
    }

    /**
     * 闭包函数
     * @param Closure $closure
     * @return mixed
     * @throws CallFailed
     */
    private function invokeFunction(Closure $closure): mixed
    {
        return $this->application->invokeFunction($closure);
    }

}
