<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 16:37
 */

use top\base\Container;
use top\base\Response;
use top\base\response\Json;
use top\exceptions\app\CallFailed;
use top\exceptions\app\ClassNotFound;
use top\base\Application;

if (!function_exists('app')) {
    /**
     * 获取App对象
     * @param string $name
     * @param array $args
     * @param bool $newInstance
     * @return Application|null
     */
    function app(string $name = 'application', array $args = [], bool $newInstance = false): ?Application
    {
        try {
            return Application::getInstance()->make($name, $args, $newInstance);
        } catch (CallFailed | ClassNotFound) {
            return null;
        }
    }
}

if (!function_exists('response')) {
    /**
     * 响应助手函数
     * @param mixed $content
     * @param string $type
     * @param int $code
     * @param array $headers
     * @return Response
     */
    function response(mixed $content, string $type = Json::class, int $code = 200, array $headers = []): Response
    {
        return Application::getInstance()->response
            ->setHeaders($headers)
            ->setHttpCode($code)
            ->setType($type)
            ->setContent($content);
    }
}

if (!function_exists('param')) {
    /**
     * 获取请求中的参数
     * @param string $name
     * @param array $except
     * @param mixed|null $default
     * @param mixed|null $filter
     * @return bool|array|null
     */
    function param(string $name = '*', array $except = [], mixed $default = null, mixed $filter = null): bool|array|null
    {
        $request    = Application::getInstance()->request;
        $parameters = $request->getParameters();
        if (str_starts_with($name, '?')) {
            $name = substr($name, 1);
            return isset($parameters[$name]);
        }
        if ($name == '*') {
            foreach ($parameters as $key => $parameter) {
                if (in_array($key, $except)) {
                    unset($parameters[$key]);
                } else {
                    // TODO 留下的参数在此处进行过滤
                    $parameters[$key] = filter($parameter, $filter);
                }
            }
            return $parameters;
        } else {
            if (isset($parameters[$name])) {
                // TODO 参数在此进行过滤
                return filter($parameters[$name], $filter);
            } else {
                return $default;
            }
        }
    }
}

if (!function_exists('filter')) {
    /**
     * 过滤数据
     * @param mixed $data
     * @param mixed|null $filter
     * @return mixed
     */
    function filter(mixed $data, mixed $filter = null): mixed
    {
        $app = Application::getInstance();
        if (is_callable($filter) || (is_string($filter) && function_exists($filter))) {
            return $app->invokeFunction($filter, [$data]);
        } elseif (is_array($filter)) {
            $class  = $filter[0] ?? '';
            $method = $filter[1] ?? '';
            return $app->invokeMethod($class, $method, [$data]);
        } else {
            return $data;
        }
    }
}

if (!function_exists('get_header')) {
    /**
     * 获取请求所有header
     * @return bool|array
     */
    function get_header(): bool|array
    {
        if (PHP_SAPI === 'apache2handler') {
            $headers = getallheaders();
            $data    = [];
            foreach ($headers as $key => $value) {
                $data[strtolower($key)] = $value;
            }
            return $data;
        } else {
            $headers = [];
            foreach ($_SERVER as $key => $value) {
                if ('http_' == strtolower(substr($key, 0, 5))) {
                    $headers[strtolower(substr($key, 5))] = $value;
                }
            }
            return $headers;
        }
    }
}
